"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StoreMeta = void 0;
const path_1 = __importDefault(require("path"));
const typescript_1 = __importDefault(require("typescript"));
const Common_1 = require("../../Common");
const Modifiers_1 = require("../AstHelpers/Modifiers");
const Imports_1 = require("../Builders/Imports");
const Object_1 = require("../Builders/Object");
const Logger_1 = require("../Logger");
const PluginConfig_1 = require("../PluginConfig");
class StoreMeta {
    constructor(file) {
        /**
         * The absolute path to the store file
         */
        Object.defineProperty(this, "absFilePath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The relative path of the store file, for ex, if importing in /src/stores/, and our file is at /src/stores/MyStore.ts, this will be MyStore.ts
         */
        Object.defineProperty(this, "relStoreFilePath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The relative import path from Stores/.generated/StoreLoader.ts to the store location
         * @type {string}
         */
        Object.defineProperty(this, "loaderImportPath", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The name of the store file
         */
        Object.defineProperty(this, "name", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * The name of the class that extends Store
         */
        Object.defineProperty(this, "className", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * This will hold the name of our store export `export const store = new MyStore();`
         */
        Object.defineProperty(this, "exportName", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        /**
         * If the dev defined `public static vueBinding = 'x';` in their store, this will hold that value
         */
        Object.defineProperty(this, "vueBinding", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "actions", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        /**
         * Method name -> handler type
         * @type {{[p: string]: string}}
         */
        Object.defineProperty(this, "lifeCycleHandlers", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "stateKeys", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "getters", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "stateObj", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.absFilePath = file.fileName;
        this.name = path_1.default.basename(file.fileName);
        this.relStoreFilePath = path_1.default.relative(PluginConfig_1.PluginConfig.generatedDir.path(), file.fileName);
        this.loaderImportPath = this.relStoreFilePath;
    }
    isValid() {
        return this.className !== undefined && this.exportName !== undefined;
    }
    finalize() {
        var _a;
        this.formatVueBinding();
        (0, Logger_1.debugLog)(`Found getters: ${(_a = this.getters) === null || _a === void 0 ? void 0 : _a.map(g => g.n).join(', ')} in ${this.className}`);
        return this;
    }
    formatVueBinding() {
        if (this.vueBinding) {
            return;
        }
        this.vueBinding = (0, Common_1.formatVueBindingName)(this.vueBinding, this.className);
    }
    metaObject(includeImport = true) {
        return {
            className: this.className,
            importPath: this.loaderImportPath,
            exportName: this.exportName,
            vueBinding: this.vueBinding,
            stateKeys: this.stateKeys,
            getters: this.getters,
            lifeCycleHandlers: this.lifeCycleHandlers,
            actions: JSON.parse(JSON.stringify(this.actions)),
            module: (0, Object_1.unwrappableNode)((0, Imports_1.createLazyImportGlobNode)(this.loaderImportPath))
        };
    }
    setStateObject(stateObj) {
        var _a;
        this.stateObj = stateObj;
        for (let property of stateObj.properties) {
            if (typescript_1.default.isPropertyAssignment(property) && (property === null || property === void 0 ? void 0 : property.name)) {
                this.stateKeys.push((_a = property.name) === null || _a === void 0 ? void 0 : _a.escapedText);
            }
        }
        (0, Logger_1.debugLog)(`Found state keys ${this.stateKeys.join(", ")} in ${this.className}`);
    }
    addGetter(member) {
        if (member.name.getText() === 'state') {
            return;
        }
        this.getters.push({
            n: member.name.getText(),
            c: (0, Modifiers_1.hasDecorator)(member, 'Computed'),
        });
    }
    addLifeCycleHandler(method, lifeCycleEvent) {
        this.lifeCycleHandlers[method.name.getText()] = lifeCycleEvent;
    }
}
exports.StoreMeta = StoreMeta;
//# sourceMappingURL=StoreMeta.js.map